# Unit tests for ax_value.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=too-many-public-methods
# pylint: disable=wrong-import-position
# pylint: disable=protected-access
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=import-outside-toplevel

"""Unit tests for ax_value.py methods."""

from __future__ import annotations

from typing import TYPE_CHECKING

import gi
import pytest

gi.require_version("Atspi", "2.0")
from gi.repository import Atspi
from gi.repository import GLib

if TYPE_CHECKING:
    from .orca_test_context import OrcaTestContext
    from unittest.mock import MagicMock

@pytest.mark.unit
class TestAXValue:
    """Test AXValue class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Returns dependencies for ax_value module testing."""

        additional_modules = ["orca.ax_utilities", "orca.ax_utilities_state"]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_tokens = test_context.Mock()
        debug_mock.LEVEL_INFO = 800

        ax_object_class_mock = test_context.Mock()
        ax_object_class_mock.supports_value = test_context.Mock(return_value=True)
        ax_object_class_mock.get_attribute = test_context.Mock(return_value="")
        essential_modules["orca.ax_object"].AXObject = ax_object_class_mock

        ax_utilities_class_mock = test_context.Mock()
        ax_utilities_class_mock.is_indeterminate = test_context.Mock(return_value=False)
        essential_modules["orca.ax_utilities"].AXUtilities = ax_utilities_class_mock

        ax_utilities_state_class_mock = test_context.Mock()
        ax_utilities_state_class_mock.is_indeterminate = test_context.Mock(
            return_value=False
        )
        essential_modules[
            "orca.ax_utilities_state"
        ].AXUtilitiesState = ax_utilities_state_class_mock

        return essential_modules

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "no_value_support",
                "supports_value": False,
                "current_value": None,
                "last_known_value": None,
                "expected_result": False,
                "setup_last_known": False,
            },
            {
                "id": "first_time_value",
                "supports_value": True,
                "current_value": 50.0,
                "last_known_value": None,
                "expected_result": True,
                "setup_last_known": False,
            },
            {
                "id": "same_value",
                "supports_value": True,
                "current_value": 50.0,
                "last_known_value": 50.0,
                "expected_result": False,
                "setup_last_known": True,
            },
            {
                "id": "value_changed",
                "supports_value": True,
                "current_value": 75.0,
                "last_known_value": 50.0,
                "expected_result": True,
                "setup_last_known": True,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_did_value_change(
        self,
        test_context,
        case: dict,
    ) -> None:
        """Test AXValue.did_value_change."""

        self._setup_dependencies(test_context)
        from orca.ax_value import AXValue
        from orca.ax_object import AXObject

        mock_accessible = test_context.Mock(spec=Atspi.Accessible)
        test_context.patch_object(
            AXObject, "supports_value", side_effect=lambda obj: case["supports_value"]
        )

        if case["current_value"] is not None:
            test_context.patch_object(
                AXValue, "_get_current_value", side_effect=lambda obj: case["current_value"]
            )

        if case["setup_last_known"]:
            AXValue.LAST_KNOWN_VALUE[hash(mock_accessible)] = case["last_known_value"]
        else:
            AXValue.LAST_KNOWN_VALUE.clear()

        result = AXValue.did_value_change(mock_accessible)
        assert result is case["expected_result"]

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "no_value_support",
                "supports_value": False,
                "atspi_return_value": None,
                "should_raise_error": False,
                "expected_result": 0.0,
            },
            {
                "id": "success_case",
                "supports_value": True,
                "atspi_return_value": 42.5,
                "should_raise_error": False,
                "expected_result": 42.5,
            },
            {
                "id": "glib_error",
                "supports_value": True,
                "atspi_return_value": None,
                "should_raise_error": True,
                "expected_result": 0.0,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_get_current_value_private(
        self,
        test_context,
        case: dict,
    ):
        """Test AXValue._get_current_value."""

        self._setup_dependencies(test_context)
        from orca.ax_value import AXValue
        from orca.ax_object import AXObject

        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        test_context.patch_object(
            AXObject, "supports_value", side_effect=lambda obj: case["supports_value"]
        )

        if case["should_raise_error"]:

            def raise_glib_error(obj):
                raise GLib.GError("Test error")

            test_context.patch_object(
                Atspi.Value, "get_current_value", side_effect=raise_glib_error
            )
        elif case["atspi_return_value"] is not None:
            test_context.patch_object(
                Atspi.Value, "get_current_value", side_effect=lambda obj: case["atspi_return_value"]
            )
        else:
            test_context.patch_object(Atspi.Value, "get_current_value", return_value=0.0)

        result = AXValue._get_current_value(mock_obj)
        assert result == case["expected_result"]

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "no_value_support",
                "supports_value": False,
                "get_current_return": 0.0,
                "expected_result": 0.0,
                "stores_value": False,
            },
            {
                "id": "stores_value",
                "supports_value": True,
                "get_current_return": 75.0,
                "expected_result": 75.0,
                "stores_value": True,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_get_current_value(
        self,
        test_context: OrcaTestContext,
        case: dict,
    ) -> None:
        """Test AXValue.get_current_value with various scenarios."""

        self._setup_dependencies(test_context)
        from orca.ax_value import AXValue
        from orca.ax_object import AXObject

        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        test_context.patch_object(
            AXObject, "supports_value", side_effect=lambda obj: case["supports_value"]
        )
        if case["supports_value"]:
            test_context.patch_object(
                AXValue, "_get_current_value", side_effect=lambda obj: case["get_current_return"]
            )
        else:
            test_context.patch_object(
                Atspi.Value, "get_current_value", side_effect=lambda obj: case["get_current_return"]
            )

        AXValue.LAST_KNOWN_VALUE.clear()
        result = AXValue.get_current_value(mock_obj)
        assert result == case["expected_result"]

        if case["stores_value"]:
            assert AXValue.LAST_KNOWN_VALUE[hash(mock_obj)] == case["expected_result"]

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "valuetext_attribute",
                "supports_value": True,
                "valuetext_attr": "75%",
                "atspi_text": "",
                "should_raise_error": False,
                "current_value": 0.0,
                "expected_result": "75%",
            },
            {
                "id": "no_value_support",
                "supports_value": False,
                "valuetext_attr": "",
                "atspi_text": "",
                "should_raise_error": False,
                "current_value": 0.0,
                "expected_result": "",
            },
            {
                "id": "from_atspi",
                "supports_value": True,
                "valuetext_attr": "",
                "atspi_text": "50 percent",
                "should_raise_error": False,
                "current_value": 0.0,
                "expected_result": "50 percent",
            },
            {
                "id": "glib_error",
                "supports_value": True,
                "valuetext_attr": "",
                "atspi_text": "",
                "should_raise_error": True,
                "current_value": 42.0,
                "expected_result": "42",
            },
            {
                "id": "decimal_places",
                "supports_value": True,
                "valuetext_attr": "",
                "atspi_text": "",
                "should_raise_error": False,
                "current_value": 0.125,
                "expected_result": "0.125",
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_get_current_value_text(  # pylint: disable=too-many-locals
        self,
        test_context: OrcaTestContext,
        case: dict,
    ) -> None:
        """Test AXValue.get_current_value_text with various scenarios."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)

        def mock_get_attribute(obj, name, check_computed=True):  # pylint: disable=unused-argument
            if name == "valuetext":
                return case["valuetext_attr"]
            return ""

        essential_modules["orca.ax_object"].AXObject.get_attribute = mock_get_attribute
        essential_modules["orca.ax_object"].AXObject.supports_value = test_context.Mock(
            return_value=case["supports_value"]
        )

        from orca.ax_value import AXValue
        from orca.ax_object import AXObject

        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        AXValue.LAST_KNOWN_VALUE.clear()

        test_context.patch_object(AXObject, "get_attribute", side_effect=mock_get_attribute)
        test_context.patch_object(
            AXObject, "supports_value", side_effect=lambda obj: case["supports_value"]
        )

        if case["should_raise_error"]:

            def raise_glib_error(self):
                raise GLib.GError("Test error")

            test_context.patch_object(Atspi.Value, "get_text", side_effect=raise_glib_error)
            test_context.patch_object(
                AXValue, "get_current_value", side_effect=lambda obj: case["current_value"]
            )
        else:
            test_context.patch_object(
                Atspi.Value, "get_text", side_effect=lambda self: case["atspi_text"]
            )
            if case["current_value"] != 0.0:
                test_context.patch_object(
                    AXValue, "get_current_value", side_effect=lambda obj: case["current_value"]
                )

        test_context.patch_object(Atspi.Value, "get_current_value", return_value=0.0)

        result = AXValue.get_current_value_text(mock_obj)
        assert result == case["expected_result"]

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "no_value_support",
                "supports_value": False,
                "current_value": None,
                "is_indeterminate": False,
                "min_val": None,
                "max_val": None,
                "expected_result": None,
            },
            {
                "id": "indeterminate_state",
                "supports_value": True,
                "current_value": 0.0,
                "is_indeterminate": True,
                "min_val": None,
                "max_val": None,
                "expected_result": None,
            },
            {
                "id": "min_equals_max",
                "supports_value": True,
                "current_value": 50.0,
                "is_indeterminate": False,
                "min_val": 100.0,
                "max_val": 100.0,
                "expected_result": None,
            },
            {
                "id": "normal_calculation",
                "supports_value": True,
                "current_value": 75.0,
                "is_indeterminate": False,
                "min_val": 0.0,
                "max_val": 100.0,
                "expected_result": 75,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_get_value_as_percent(
        self,
        test_context,
        case: dict,
    ) -> None:
        """Test AXValue.get_value_as_percent."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)

        supports_value_mock = test_context.Mock(return_value=case["supports_value"])
        is_indeterminate_mock = test_context.Mock(return_value=case["is_indeterminate"])
        essential_modules["orca.ax_object"].AXObject.supports_value = supports_value_mock
        essential_modules["orca.ax_utilities"].AXUtilities.is_indeterminate = is_indeterminate_mock

        from orca.ax_value import AXValue
        from orca.ax_object import AXObject
        from orca.ax_utilities import AXUtilities

        mock_obj = test_context.Mock(spec=Atspi.Accessible)

        AXValue.LAST_KNOWN_VALUE.clear()

        test_context.patch_object(
            AXObject, "supports_value", side_effect=lambda obj: case["supports_value"]
        )

        current_val = case["current_value"] if case["current_value"] is not None else 0.0
        min_val_default = case["min_val"] if case["min_val"] is not None else 0.0
        max_val_default = case["max_val"] if case["max_val"] is not None else 100.0

        test_context.patch_object(
            Atspi.Value, "get_current_value", side_effect=lambda obj: current_val
        )
        test_context.patch_object(
            Atspi.Value, "get_minimum_value", side_effect=lambda obj: min_val_default
        )
        test_context.patch_object(
            Atspi.Value, "get_maximum_value", side_effect=lambda obj: max_val_default
        )

        test_context.patch_object(
            AXUtilities, "is_indeterminate", side_effect=lambda obj: case["is_indeterminate"]
        )

        if case["current_value"] is not None:
            test_context.patch_object(
                AXValue, "get_current_value", side_effect=lambda obj: case["current_value"]
            )
        if case["min_val"] is not None:
            test_context.patch_object(
                AXValue, "get_minimum_value", side_effect=lambda obj: case["min_val"]
            )
        if case["max_val"] is not None:
            test_context.patch_object(
                AXValue, "get_maximum_value", side_effect=lambda obj: case["max_val"]
            )

        result = AXValue.get_value_as_percent(mock_obj)
        assert result == case["expected_result"]

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "no_value_support",
                "supports_value": False,
                "current_value": None,
                "cached_value": None,
                "expected_result": False,
                "expects_debug_call": False,
                "cache_should_exist": False,
            },
            {
                "id": "cached_value_changed",
                "supports_value": True,
                "current_value": 75.0,
                "cached_value": 50.0,
                "expected_result": True,
                "expects_debug_call": True,
                "cache_should_exist": True,
            },
            {
                "id": "cached_value_unchanged",
                "supports_value": True,
                "current_value": 50.0,
                "cached_value": 50.0,
                "expected_result": False,
                "expects_debug_call": False,
                "cache_should_exist": True,
            },
            {
                "id": "no_cached_value",
                "supports_value": True,
                "current_value": 25.0,
                "cached_value": None,
                "expected_result": True,
                "expects_debug_call": True,
                "cache_should_exist": False,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_did_value_change_scenarios(
        self,
        test_context: OrcaTestContext,
        case: dict,
    ) -> None:
        """Test AXValue.did_value_change with various scenarios."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.ax_value import AXValue
        from orca.ax_object import AXObject

        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        test_context.patch_object(
            AXObject, "supports_value", side_effect=lambda obj: case["supports_value"]
        )

        if case["current_value"] is not None:
            test_context.patch_object(
                AXValue, "_get_current_value", side_effect=lambda obj: case["current_value"]
            )

        AXValue.LAST_KNOWN_VALUE.clear()
        if case["cached_value"] is not None:
            AXValue.LAST_KNOWN_VALUE[hash(mock_obj)] = case["cached_value"]

        result = AXValue.did_value_change(mock_obj)
        assert result is case["expected_result"]

        if case["expects_debug_call"]:
            essential_modules["orca.debug"].print_tokens.assert_called()

        if case["cache_should_exist"] and case["cached_value"] is not None:
            assert AXValue.LAST_KNOWN_VALUE[hash(mock_obj)] == case["cached_value"]
        elif not case["cache_should_exist"]:
            assert hash(mock_obj) not in AXValue.LAST_KNOWN_VALUE

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "min_no_value_support",
                "method_name": "get_minimum_value",
                "supports_value": False,
                "atspi_return_value": None,
                "should_raise_error": False,
                "expected_result": 0.0,
            },
            {
                "id": "min_success_case",
                "method_name": "get_minimum_value",
                "supports_value": True,
                "atspi_return_value": 10.0,
                "should_raise_error": False,
                "expected_result": 10.0,
            },
            {
                "id": "min_glib_error",
                "method_name": "get_minimum_value",
                "supports_value": True,
                "atspi_return_value": None,
                "should_raise_error": True,
                "expected_result": 0.0,
            },
            {
                "id": "min_negative_values",
                "method_name": "get_minimum_value",
                "supports_value": True,
                "atspi_return_value": -50.5,
                "should_raise_error": False,
                "expected_result": -50.5,
            },
            {
                "id": "max_no_value_support",
                "method_name": "get_maximum_value",
                "supports_value": False,
                "atspi_return_value": None,
                "should_raise_error": False,
                "expected_result": 0.0,
            },
            {
                "id": "max_success_case",
                "method_name": "get_maximum_value",
                "supports_value": True,
                "atspi_return_value": 100.0,
                "should_raise_error": False,
                "expected_result": 100.0,
            },
            {
                "id": "max_glib_error",
                "method_name": "get_maximum_value",
                "supports_value": True,
                "atspi_return_value": None,
                "should_raise_error": True,
                "expected_result": 0.0,
            },
            {
                "id": "max_large_values",
                "method_name": "get_maximum_value",
                "supports_value": True,
                "atspi_return_value": 99999.99,
                "should_raise_error": False,
                "expected_result": 99999.99,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_get_minimum_maximum_value(
        self,
        test_context: OrcaTestContext,
        case: dict,
    ) -> None:
        """Test AXValue.get_minimum_value and get_maximum_value with various scenarios."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.ax_value import AXValue
        from orca.ax_object import AXObject

        mock_obj = test_context.Mock(spec=Atspi.Accessible)
        test_context.patch_object(
            AXObject, "supports_value", side_effect=lambda obj: case["supports_value"]
        )

        if case["should_raise_error"]:

            def raise_glib_error(obj):  # pylint: disable=unused-argument
                error_type = "minimum" if "minimum" in case["method_name"] else "maximum"
                raise GLib.GError(f"Test {error_type} value error")

            test_context.patch_object(
                Atspi.Value, case["method_name"], side_effect=raise_glib_error
            )
        elif case["atspi_return_value"] is not None:
            test_context.patch_object(
                Atspi.Value, case["method_name"], side_effect=lambda obj: case["atspi_return_value"]
            )
        else:
            test_context.patch_object(Atspi.Value, case["method_name"], return_value=0.0)

        result = getattr(AXValue, case["method_name"])(mock_obj)
        assert result == case["expected_result"]

        if case["supports_value"]:
            if case["should_raise_error"]:
                essential_modules["orca.debug"].print_message.assert_called()
            else:
                essential_modules["orca.debug"].print_tokens.assert_called()



#!/usr/bin/python
# dump-window-events.py
#
# Command-line tool to dump window activation and deactivation events along with
# the window(s) which claim to be active.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=invalid-name
# pylint: disable=wrong-import-position

"""Command-line tool to dump window activation and deactivation events."""

import gi
gi.require_version("Atspi", "2.0")
from gi.repository import Atspi, GLib

def clear_atspi_cache(obj):
    """Clear the AT-SPI cache for a specific object."""

    try:
        Atspi.Accessible.clear_cache_single(obj)
    except GLib.GError:
        pass
    else:
        return

    try:
        Atspi.Accessible.clear_cache(obj)
    except GLib.GError:
        pass
    else:
        return

def get_all_applications():
    """Get all running accessible applications."""

    apps = []
    desktop = Atspi.get_desktop(0)
    for i in range(Atspi.Accessible.get_child_count(desktop)):
        try:
            child = Atspi.Accessible.get_child_at_index(desktop, i)
            # Children of mutter-x11-frames seem to mirror the actual apps, but are not what we
            # want. And Orca ignores them because we need to handle the real app; not some clone.
            if Atspi.Accessible.get_name(child) != "mutter-x11-frames":
                apps.append(child)
        except GLib.GError:
            continue

    return apps

def find_all_active_windows(apps=None):
    """Find all windows which claim to be active.."""

    active_windows = []
    if apps is None:
        apps = get_all_applications()

    for app in apps:
        try:
            for i in range(Atspi.Accessible.get_child_count(app)):
                child = Atspi.Accessible.get_child_at_index(app, i)
                # To ensure we're not basing the decision on stale data.
                clear_atspi_cache(child)
                state_set = Atspi.Accessible.get_state_set(child)
                if state_set.contains(Atspi.StateType.ACTIVE):
                    active_windows.append(child)
        except GLib.GError as error:
            print(f"(error finding active windows: {error})")

    return active_windows

def get_name_and_role(obj):
    """Get the name and role of an accessible object."""

    try:
        name = Atspi.Accessible.get_name(obj) or "<unnamed>"
        role = Atspi.Accessible.get_role(obj).value_nick
        return f"'{name}' {role}"
    except GLib.GError as error:
        return f"(error getting name and role: {error})"

def get_top_level_object(obj):
    """Get the highest level accessible object for a given object."""

    # In well-behaved applications, this should work.
    try:
        application = Atspi.Accessible.get_application(obj)
    except GLib.GError as error:
        print(f"(error getting application: {error})")
    else:
        if application is not None:
            return application

    # Walk up the tree as far as we can. We should be able to ascend all the way to the app.
    # But if we wound up here, we already have breakage and knowing how far we got may help
    # with debugging.
    desktop = Atspi.get_desktop(0)
    result = obj
    while result is not None:
        try:
            parent = Atspi.Accessible.get_parent(result)
        except GLib.GError:
            break
        if parent in (None, desktop):
            break
        result = parent

    return result

def get_object_info(obj):
    """Returns basic info about an accessible object."""

    name_and_role = get_name_and_role(obj)
    top_level_name_and_role = get_name_and_role(get_top_level_object(obj))
    return f"{name_and_role} from: {top_level_name_and_role}"

def on_window_event(e):
    """Handle window activation and deactivation events."""

    event_type = e.type.split(":")[-1].upper()
    print(f"\n{event_type}: {get_object_info(e.source)}")

    apps = get_all_applications()
    active_windows = find_all_active_windows(apps)
    if not active_windows:
        print("No windows have the active state. The following applications are known to AT-SPI2:")
        for i, app in enumerate(apps):
            try:
                print(f"{i+1:2}. {Atspi.Accessible.get_name(app)}")
            except GLib.GError as error:
                print(f"{i+1:2}. {error}")
        return

    if len(active_windows) == 1:
        print(f"The active window is: {get_object_info(active_windows[0])}")
        return

    print(f"{len(active_windows)} windows claim they are active:")
    for i, win in enumerate(active_windows):
        try:
            print(f"{i+1:2}. {get_object_info(win)}")
        except GLib.GError as error:
            print(f"{i+1:2}. {error}")

def on_focus_event(e):
    """Handle focus events for exit condition."""

    if not e.detail1:
        return

    try:
        if Atspi.Accessible.get_role(e.source) == Atspi.Role.TERMINAL:
            print("Exiting.")
            Atspi.event_quit()
    except GLib.GError:
        pass

def main():
    """Starts the window event dumper and waits for events."""

    window_listener = Atspi.EventListener.new(on_window_event)
    window_listener.register("window:activate")
    window_listener.register("window:deactivate")

    focus_listener = Atspi.EventListener.new(on_focus_event)
    focus_listener.register("object:state-changed:focused")

    print("Listening for window activation/deactivation events. Focus a terminal to exit.")
    Atspi.event_main()

if __name__ == "__main__":
    main()



# Unit tests for script_manager.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-statements
# pylint: disable=too-few-public-methods
# pylint: disable=protected-access
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=too-many-locals
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines

"""Unit tests for script_manager.py methods."""

from __future__ import annotations

import sys
from typing import TYPE_CHECKING
from unittest.mock import call, Mock

import pytest

if TYPE_CHECKING:
    from .orca_test_context import OrcaTestContext
    from unittest.mock import MagicMock

@pytest.mark.unit
class TestScriptManager:
    """Test ScriptManager class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Returns dependencies for script_manager module testing."""

        modules_to_clean = ["orca.script_manager"]
        for module_name in modules_to_clean:
            if module_name in sys.modules:
                del sys.modules[module_name]

        additional_modules = [
            "gi",
            "gi.repository",
            "gi.repository.Atspi",
            "orca.ax_utilities",
            "orca.scripts",
            "orca.scripts.apps",
            "orca.scripts.default",
            "orca.scripts.sleepmode",
            "orca.scripts.toolkits",
            "orca.speech_and_verbosity_manager",
        ]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        if "orca.script_manager" in essential_modules:
            del essential_modules["orca.script_manager"]
        if "orca.script_manager" in sys.modules:
            del sys.modules["orca.script_manager"]

        gi_repository_mock = essential_modules["gi.repository"]
        atspi_mock = essential_modules["gi.repository.Atspi"]

        class UnionSupportingMock(Mock):
            """Mock that supports union operations for Atspi.Accessible."""

            def __or__(self, other):
                return UnionSupportingMock()

        atspi_mock.Accessible = UnionSupportingMock()
        gi_repository_mock.Atspi = atspi_mock

        ax_object_mock = essential_modules["orca.ax_object"]
        ax_object_class_mock = test_context.Mock()
        ax_object_class_mock.get_name = test_context.Mock(return_value="test-app")
        ax_object_class_mock.get_attribute = test_context.Mock(return_value="GTK")
        ax_object_mock.AXObject = ax_object_class_mock

        ax_utilities_mock = essential_modules["orca.ax_utilities"]
        ax_utilities_class_mock = test_context.Mock()
        ax_utilities_class_mock.is_terminal = test_context.Mock(return_value=False)
        ax_utilities_class_mock.get_application_toolkit_name = test_context.Mock(
            return_value="gtk"
        )
        ax_utilities_class_mock.is_application_in_desktop = test_context.Mock(
            return_value=True
        )
        ax_utilities_class_mock.is_frame = test_context.Mock(return_value=False)
        ax_utilities_class_mock.is_status_bar = test_context.Mock(return_value=False)
        ax_utilities_mock.AXUtilities = ax_utilities_class_mock

        braille_mock = essential_modules["orca.braille"]
        braille_mock.checkBrailleSetting = test_context.Mock()
        braille_mock.setupKeyRanges = test_context.Mock()

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_message = test_context.Mock()
        debug_mock.LEVEL_INFO = 800

        settings_manager_mock = essential_modules["orca.settings_manager"]
        settings_manager_instance = test_context.Mock()
        settings_manager_instance.get_runtime_settings = test_context.Mock(return_value={})
        settings_manager_instance.set_setting = test_context.Mock()
        settings_manager_mock.get_manager = test_context.Mock(
            return_value=settings_manager_instance
        )

        speech_verbosity_mock = essential_modules["orca.speech_and_verbosity_manager"]
        speech_manager_instance = test_context.Mock()
        speech_manager_instance.check_speech_setting = test_context.Mock()
        speech_verbosity_mock.get_manager = test_context.Mock(
            return_value=speech_manager_instance
        )

        scripts_mock = essential_modules["orca.scripts"]
        apps_mock = essential_modules["orca.scripts.apps"]
        toolkits_mock = essential_modules["orca.scripts.toolkits"]
        default_mock = essential_modules["orca.scripts.default"]
        sleepmode_mock = essential_modules["orca.scripts.sleepmode"]

        apps_mock.__all__ = ["evolution", "gedit", "gnome-shell"]
        toolkits_mock.__all__ = ["gtk", "Gecko", "Qt"]

        class MockScript:
            """Mock script class for testing."""

            def __init__(self, app=None):
                self.app = app
                self.register_event_listeners = test_context.Mock()
                self.deregister_event_listeners = test_context.Mock()
                self.activate = test_context.Mock()
                self.deactivate = test_context.Mock()
                self.braille_bindings = {}
                self._sleep_mode_manager = test_context.Mock()
                self._sleep_mode_manager.is_active_for_app = test_context.Mock(
                    return_value=False
                )
                self.get_sleep_mode_manager = test_context.Mock(
                    return_value=self._sleep_mode_manager
                )

            def __or__(self, other):
                return MockScript

        default_script_constructor = test_context.Mock(return_value=MockScript())
        default_script_constructor.__or__ = lambda self, other: MockScript()
        sleepmode_script_constructor = test_context.Mock(return_value=MockScript())
        sleepmode_script_constructor.__or__ = lambda self, other: MockScript()

        default_mock.Script = default_script_constructor
        sleepmode_mock.Script = sleepmode_script_constructor

        scripts_mock.apps = apps_mock
        scripts_mock.default = default_mock
        scripts_mock.sleepmode = sleepmode_mock
        scripts_mock.toolkits = toolkits_mock

        essential_modules["settings_manager_instance"] = settings_manager_instance
        essential_modules["speech_manager_instance"] = speech_manager_instance

        default_script = test_context.Mock()
        default_script.app = test_context.Mock()
        default_script.register_event_listeners = test_context.Mock()
        default_script.deregister_event_listeners = test_context.Mock()
        default_script.activate = test_context.Mock()
        default_script.deactivate = test_context.Mock()
        default_script.braille_bindings = {}

        sleepmode_script = test_context.Mock()
        sleepmode_script.app = test_context.Mock()
        sleepmode_script.register_event_listeners = test_context.Mock()
        sleepmode_script.deregister_event_listeners = test_context.Mock()
        sleepmode_script.activate = test_context.Mock()
        sleepmode_script.deactivate = test_context.Mock()

        sleep_mode_manager = test_context.Mock()
        sleep_mode_manager.is_active_for_app = test_context.Mock(return_value=False)

        essential_modules["default_script"] = default_script
        essential_modules["sleepmode_script"] = sleepmode_script
        essential_modules["sleep_mode_manager"] = sleep_mode_manager

        default_module = test_context.Mock()
        default_module.Script = test_context.Mock(return_value=test_context.Mock())

        sleepmode_module = test_context.Mock()
        sleepmode_module.Script = test_context.Mock(return_value=test_context.Mock())

        deps_settings_manager = test_context.Mock()
        deps_settings_manager_instance = test_context.Mock()
        deps_settings_manager_instance.get_runtime_settings = test_context.Mock(return_value={})
        deps_settings_manager_instance.set_setting = test_context.Mock()
        deps_settings_manager.get_manager = test_context.Mock(
            return_value=deps_settings_manager_instance
        )

        deps_speech_manager = test_context.Mock()
        deps_speech_manager_instance = test_context.Mock()
        deps_speech_manager_instance.check_speech_setting = test_context.Mock()
        deps_speech_manager.get_manager = test_context.Mock(
            return_value=deps_speech_manager_instance
        )

        braille_module = test_context.Mock()
        braille_module.checkBrailleSetting = test_context.Mock()
        braille_module.setupKeyRanges = test_context.Mock()

        essential_modules["default_module"] = default_module
        essential_modules["sleepmode_module"] = sleepmode_module
        essential_modules["deps_settings_manager"] = deps_settings_manager
        essential_modules["deps_speech_manager"] = deps_speech_manager
        essential_modules["braille_module"] = braille_module

        return essential_modules


    def test_init(self, test_context: OrcaTestContext) -> None:
        """Test ScriptManager.__init__."""

        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        assert not manager.app_scripts
        assert not manager.toolkit_scripts
        assert not manager.custom_scripts
        assert not manager._sleep_mode_scripts
        assert manager._default_script is None
        assert manager._active_script is None
        assert manager._active is False

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "inactive_to_active",
                "initially_active": False,
                "expects_script_creation": True,
            },
            {"id": "already_active", "initially_active": True, "expects_script_creation": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_activate(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test ScriptManager.activate with different initial states."""

        essential_modules = self._setup_dependencies(test_context)
        default_script = essential_modules["default_script"]

        if case["expects_script_creation"]:
            mock_default_script = test_context.Mock()
            test_context.patch(
                "orca.script_manager.default.Script", new=mock_default_script
            )
            mock_default_script.return_value = default_script

        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        manager._active = case["initially_active"]
        manager.activate()

        assert manager._active is True

        if case["expects_script_creation"]:
            assert manager._default_script is not None
            assert manager._active_script is not None
            default_script.register_event_listeners.assert_called_once()

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "active_to_inactive", "initially_active": True, "expects_cleanup": True},
            {"id": "already_inactive", "initially_active": False, "expects_cleanup": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_deactivate(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test ScriptManager.deactivate with different initial states."""

        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        manager._active = case["initially_active"]

        if case["expects_cleanup"]:
            mock_script = test_context.Mock()
            mock_script.deregister_event_listeners = test_context.Mock()
            manager._default_script = mock_script
            manager.app_scripts = {"test": "script"}
            manager.toolkit_scripts = {"test": "script"}
            manager.custom_scripts = {"test": "script"}

        manager.deactivate()
        assert manager._active is False

        if case["expects_cleanup"]:
            assert manager._default_script is None
            assert manager._active_script is None
            assert not manager.app_scripts
            assert not manager.toolkit_scripts
            assert not manager.custom_scripts
            mock_script.deregister_event_listeners.assert_called_once()

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "app_in_apps_list",
                "app_name": "evolution",
                "expected_result": "evolution",
                "use_null_app": False,
            },
            {
                "id": "toolkit_in_toolkits_list",
                "app_name": "gtk",
                "expected_result": "gtk",
                "use_null_app": False,
            },
            {
                "id": "mapped_app_name",
                "app_name": "mate-notification-daemon",
                "expected_result": "notification-daemon",
                "use_null_app": False,
            },
            {
                "id": "alias_app_name",
                "app_name": "pluma",
                "expected_result": "gedit",
                "use_null_app": False,
            },
            {
                "id": "python_extension",
                "app_name": "test-app.py",
                "expected_result": "test-app",
                "use_null_app": False,
            },
            {
                "id": "bin_extension",
                "app_name": "test-app.bin",
                "expected_result": "test-app",
                "use_null_app": False,
            },
            {
                "id": "reverse_domain",
                "app_name": "org.gnome.TestApp",
                "expected_result": "TestApp",
                "use_null_app": False,
            },
            {
                "id": "reverse_domain_com",
                "app_name": "com.example.TestApp",
                "expected_result": "TestApp",
                "use_null_app": False,
            },
            {
                "id": "unknown_app",
                "app_name": "unknown-app",
                "expected_result": "unknown-app",
                "use_null_app": False,
            },
            {"id": "null_app", "app_name": None, "expected_result": None, "use_null_app": True},
            {"id": "nameless_app", "app_name": "", "expected_result": None, "use_null_app": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_module_name_scenarios(self, test_context, case: dict) -> None:
        """Test ScriptManager.get_module_name with various scenarios."""
        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()

        if case["use_null_app"]:
            result = manager.get_module_name(None)
        else:
            mock_app = test_context.Mock()
            mock_ax_object = test_context.Mock()
            test_context.patch("orca.script_manager.AXObject", new=mock_ax_object)
            mock_ax_object.get_name.return_value = case["app_name"]
            result = manager.get_module_name(mock_app)

        assert result == case["expected_result"]

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "gtk_toolkit", "toolkit_attribute": "GTK", "expected_result": "gtk"},
            {"id": "gail_mapped_to_gtk", "toolkit_attribute": "GAIL", "expected_result": "gtk"},
            {"id": "qt_toolkit", "toolkit_attribute": "Qt", "expected_result": "Qt"},
            {"id": "empty_toolkit", "toolkit_attribute": "", "expected_result": ""},
            {"id": "none_toolkit", "toolkit_attribute": None, "expected_result": None},
        ],
        ids=lambda case: case["id"],
    )
    def test_toolkit_for_object(self, test_context, case: dict) -> None:
        """Test ScriptManager._toolkit_for_object."""

        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_obj = test_context.Mock()

        mock_ax_object = test_context.Mock()
        test_context.patch("orca.script_manager.AXObject", new=mock_ax_object)
        mock_ax_object.get_attribute.return_value = case["toolkit_attribute"]
        result = manager._toolkit_for_object(mock_obj)
        assert result == case["expected_result"]
        mock_ax_object.get_attribute.assert_called_once_with(mock_obj, "toolkit")

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "terminal_role", "is_terminal": True, "expected_result": "terminal"},
            {"id": "non_terminal_role", "is_terminal": False, "expected_result": ""},
        ],
        ids=lambda case: case["id"],
    )
    def test_script_for_role(self, test_context, case: dict) -> None:
        """Test ScriptManager._script_for_role."""

        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_obj = test_context.Mock()

        mock_ax_utilities = test_context.Mock()
        test_context.patch("orca.script_manager.AXUtilities", new=mock_ax_utilities)
        mock_ax_utilities.is_terminal.return_value = case["is_terminal"]
        result = manager._script_for_role(mock_obj)
        assert result == case["expected_result"]
        mock_ax_utilities.is_terminal.assert_called_once_with(mock_obj)

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "null_app_empty_name",
                "app": None,
                "name": "",
                "has_module": False,
                "has_get_script": False,
                "has_script_class": False,
                "should_succeed": False,
            },
            {
                "id": "empty_name",
                "app": "app",
                "name": "",
                "has_module": False,
                "has_get_script": False,
                "has_script_class": False,
                "should_succeed": False,
            },
            {
                "id": "no_module",
                "app": "app",
                "name": "test",
                "has_module": False,
                "has_get_script": False,
                "has_script_class": False,
                "should_succeed": False,
            },
            {
                "id": "has_get_script",
                "app": "app",
                "name": "test",
                "has_module": True,
                "has_get_script": True,
                "has_script_class": False,
                "should_succeed": True,
            },
            {
                "id": "has_script_class",
                "app": "app",
                "name": "test",
                "has_module": True,
                "has_get_script": False,
                "has_script_class": True,
                "should_succeed": True,
            },
            {
                "id": "no_script_creation",
                "app": "app",
                "name": "test",
                "has_module": True,
                "has_get_script": False,
                "has_script_class": False,
                "should_succeed": False,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_new_named_script(self, test_context, case: dict) -> None:
        """Test ScriptManager._new_named_script."""

        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_app = test_context.Mock() if case["app"] else None
        mock_script = test_context.Mock()
        mock_import = test_context.Mock()
        test_context.patch("importlib.import_module", side_effect=mock_import)
        if case["has_module"]:
            mock_module = type("MockModule", (), {})()
            if case["has_get_script"]:
                mock_module.get_script = test_context.Mock(return_value=mock_script)
            elif case["has_script_class"]:
                mock_module.Script = test_context.Mock(return_value=mock_script)
            mock_import.return_value = mock_module
        else:
            mock_import.side_effect = ImportError("Module not found")
        result = manager._new_named_script(mock_app, case["name"])
        if case["should_succeed"]:
            assert result == mock_script
        else:
            assert result is None

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "os_error_causes_unboundlocal",
                "exception_type": "OSError",
            },
            {
                "id": "script_creation_error",
                "exception_type": "AttributeError",
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_new_named_script_error_handling(
        self, test_context: OrcaTestContext, case: dict
    ) -> None:
        """Test ScriptManager._new_named_script handles various errors."""

        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_app = test_context.Mock()
        mock_import = test_context.Mock()
        test_context.patch("importlib.import_module", side_effect=mock_import)

        if case["exception_type"] == "OSError":
            mock_import.side_effect = OSError("Permission denied")
            with pytest.raises(UnboundLocalError):
                manager._new_named_script(mock_app, "test")
        else:
            mock_module = type("MockModule", (), {})()
            mock_module.Script = test_context.Mock(
                side_effect=AttributeError("Script class not found")
            )
            mock_import.return_value = mock_module
            result = manager._new_named_script(mock_app, "test")
            assert result is None

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "no_module_name", "module_name": None, "expected_result_type": "default_script"},
            {
                "id": "with_module_name",
                "module_name": "test_script",
                "expected_result_type": "named_script",
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_create_script(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test ScriptManager._create_script with different module name scenarios."""

        essential_modules = self._setup_dependencies(test_context)
        default_script = essential_modules["default_script"]
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_app = test_context.Mock()
        mock_obj = test_context.Mock()
        mock_script = test_context.Mock()

        test_context.patch_object(
            manager, "get_module_name", return_value=case["module_name"]
        )
        test_context.patch_object(
            manager, "_toolkit_for_object", return_value=None
        )
        test_context.patch_object(
            manager, "get_default_script", return_value=default_script
        )

        test_context.patch(
            "orca.script_manager.AXUtilities.get_application_toolkit_name",
            return_value=None,
        )

        if case["expected_result_type"] == "default_script":
            mock_new_named_script = test_context.Mock(return_value=None)
        else:
            mock_new_named_script = test_context.Mock(return_value=mock_script)
        test_context.patch_object(manager, "_new_named_script", side_effect=mock_new_named_script)

        result = manager._create_script(mock_app, mock_obj)

        if case["expected_result_type"] == "default_script":
            assert result == default_script
        else:
            assert result == mock_script
            mock_new_named_script.assert_called_once_with(mock_app, case["module_name"])

    def test_get_default_script(self, test_context: OrcaTestContext) -> None:
        """Test ScriptManager.get_default_script."""

        essential_modules = self._setup_dependencies(test_context)
        default_script = essential_modules["default_script"]
        from orca.script_manager import ScriptManager

        mock_default_script = test_context.Mock()
        test_context.patch("orca.script_manager.default.Script", new=mock_default_script)
        mock_default_script.return_value = default_script
        manager = ScriptManager()
        mock_app = test_context.Mock()

        result = manager.get_default_script(mock_app)
        assert result is not None
        mock_default_script.assert_called_with(mock_app)

        manager._default_script = default_script
        result = manager.get_default_script(None)
        assert result == default_script

        manager._default_script = None
        result = manager.get_default_script(None)
        assert result is not None
        assert manager._default_script is not None

    def test_get_or_create_sleep_mode_script(self, test_context: OrcaTestContext) -> None:
        """Test ScriptManager.get_or_create_sleep_mode_script."""

        essential_modules = self._setup_dependencies(test_context)
        sleepmode_script = essential_modules["sleepmode_script"]
        from orca.script_manager import ScriptManager

        mock_sleepmode_script = test_context.Mock()
        test_context.patch(
            "orca.script_manager.sleepmode.Script", new=mock_sleepmode_script
        )
        mock_sleepmode_script.return_value = sleepmode_script
        manager = ScriptManager()
        mock_app = test_context.Mock()

        result = manager.get_or_create_sleep_mode_script(mock_app)
        assert result is not None
        assert result == sleepmode_script
        assert manager._sleep_mode_scripts[mock_app] == result

        result2 = manager.get_or_create_sleep_mode_script(mock_app)
        assert result2 == result
        assert len(manager._sleep_mode_scripts) == 1

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "null_app_and_obj",
                "app": None,
                "obj": None,
                "sleep_mode_active": False,
                "has_custom_script": False,
                "has_toolkit_script": False,
                "expected_script_type": "default",
            },
            {
                "id": "sleep_mode_active",
                "app": "app",
                "obj": "obj",
                "sleep_mode_active": True,
                "has_custom_script": False,
                "has_toolkit_script": False,
                "expected_script_type": "sleep",
            },
            {
                "id": "custom_script",
                "app": "app",
                "obj": "obj",
                "sleep_mode_active": False,
                "has_custom_script": True,
                "has_toolkit_script": False,
                "expected_script_type": "custom",
            },
            {
                "id": "toolkit_script",
                "app": "app",
                "obj": "obj",
                "sleep_mode_active": False,
                "has_custom_script": False,
                "has_toolkit_script": True,
                "expected_script_type": "toolkit",
            },
            {
                "id": "app_script",
                "app": "app",
                "obj": "obj",
                "sleep_mode_active": False,
                "has_custom_script": False,
                "has_toolkit_script": False,
                "expected_script_type": "app",
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_get_script(self, test_context, case: dict) -> None:
        """Test ScriptManager.get_script."""

        essential_modules = self._setup_dependencies(test_context)
        sleepmode_script = essential_modules["sleepmode_script"]
        sleep_mode_manager = essential_modules["sleep_mode_manager"]
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_app = test_context.Mock() if case["app"] else None
        mock_obj = test_context.Mock() if case["obj"] else None
        mock_sleep_script = sleepmode_script
        mock_custom_script = test_context.Mock()

        class ToolkitScript:
            """Mock toolkit script class."""

        class AppScript:
            """Mock app script class."""

        mock_toolkit_script = test_context.Mock(spec=ToolkitScript)
        mock_app_script = test_context.Mock(spec=AppScript)
        sleep_mode_manager.is_active_for_app = test_context.Mock(
            return_value=case["sleep_mode_active"]
        )
        mock_app_script.get_sleep_mode_manager = test_context.Mock(return_value=sleep_mode_manager)
        test_context.patch_object(
            manager,
            "get_or_create_sleep_mode_script",
            return_value=mock_sleep_script,
        )
        test_context.patch_object(
            manager,
            "_script_for_role",
            return_value="terminal" if case["has_custom_script"] else "",
        )
        test_context.patch_object(
            manager,
            "_toolkit_for_object",
            return_value="gtk" if case["has_toolkit_script"] else None,
        )

        def create_script_side_effect(_app, obj):
            if obj and case["has_toolkit_script"]:
                return mock_toolkit_script
            return mock_app_script

        test_context.patch_object(
            manager, "_create_script", side_effect=create_script_side_effect
        )
        if case["has_custom_script"]:
            test_context.patch_object(
                manager, "_new_named_script", return_value=mock_custom_script
            )
        result = manager.get_script(mock_app, mock_obj)
        if case["expected_script_type"] == "default":
            assert result is not None
            assert hasattr(result, "register_event_listeners")
        elif case["expected_script_type"] == "sleep":
            assert result == mock_sleep_script
        elif case["expected_script_type"] == "custom":
            assert result == mock_custom_script
        elif case["expected_script_type"] == "toolkit":
            assert result == mock_toolkit_script
        elif case["expected_script_type"] == "app":
            assert result == mock_app_script

    def test_get_script_exception_handling(self, test_context: OrcaTestContext) -> None:
        """Test ScriptManager.get_script handles exceptions."""

        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_app = test_context.Mock()
        mock_obj = test_context.Mock()
        test_context.patch_object(
            manager, "_script_for_role", return_value=""
        )
        test_context.patch_object(
            manager, "_toolkit_for_object", return_value=None
        )

        def create_script_side_effect(app, obj):
            raise KeyError("Script creation failed")

        test_context.patch_object(
            manager, "_create_script", side_effect=create_script_side_effect
        )
        result = manager.get_script(mock_app, mock_obj)
        assert result is not None
        assert hasattr(result, "register_event_listeners")

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "no_active_script", "has_active_script": False, "test_type": "script"},
            {"id": "has_active_script", "has_active_script": True, "test_type": "script"},
            {"id": "no_active_script_app", "has_active_script": False, "test_type": "app"},
            {"id": "has_active_script_app", "has_active_script": True, "test_type": "app"},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_active_script_and_app(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test ScriptManager.get_active_script and get_active_script_app."""
        essential_modules = self._setup_dependencies(test_context)
        default_script = essential_modules["default_script"]
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        mock_app = test_context.Mock()

        if case["has_active_script"]:
            default_script.app = mock_app
            manager._active_script = default_script

        if case["test_type"] == "script":
            result = manager.get_active_script()
            expected = default_script if case["has_active_script"] else None
        else:
            result = manager.get_active_script_app()
            expected = mock_app if case["has_active_script"] else None

        assert result == expected

    def test_set_active_script(self, test_context: OrcaTestContext) -> None:
        """Test ScriptManager.set_active_script."""

        essential_modules = self._setup_dependencies(test_context)
        default_script = essential_modules["default_script"]
        settings_manager = essential_modules["deps_settings_manager"]
        from orca.script_manager import ScriptManager

        mock_check_braille = test_context.Mock()
        test_context.patch(
            "orca.script_manager.braille.checkBrailleSetting", new=mock_check_braille
        )
        mock_setup_ranges = test_context.Mock()
        test_context.patch(
            "orca.script_manager.braille.setupKeyRanges", new=mock_setup_ranges
        )
        mock_get_speech_manager = test_context.Mock()
        test_context.patch(
            "orca.script_manager.speech_and_verbosity_manager.get_manager",
            new=mock_get_speech_manager,
        )
        mock_speech_manager_instance = test_context.Mock()
        mock_get_speech_manager.return_value = mock_speech_manager_instance
        manager = ScriptManager()
        old_script = test_context.Mock()
        old_script.app = test_context.Mock()
        old_script.deactivate = test_context.Mock()
        new_script = default_script
        new_script.app = test_context.Mock()
        new_script.activate = test_context.Mock()
        new_script.braille_bindings = {"key": "binding"}
        manager_instance = test_context.Mock()
        manager_instance.get_runtime_settings = test_context.Mock(return_value={"setting": "value"})
        manager_instance.set_setting = test_context.Mock()
        settings_manager.get_manager = test_context.Mock(return_value=manager_instance)

        manager._active_script = old_script
        manager.set_active_script(new_script, "test reason")
        old_script.deactivate.assert_called_once()
        new_script.activate.assert_called_once()
        assert manager._active_script == new_script

        mock_check_braille.assert_called_once()
        mock_setup_ranges.assert_called_once_with(new_script.braille_bindings.keys())
        mock_speech_manager_instance.check_speech_setting.assert_called_once()

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "same_script_no_change",
                "new_script_type": "same",
                "expects_deactivate": False,
                "expects_activate": False,
            },
            {
                "id": "set_to_none",
                "new_script_type": "none",
                "expects_deactivate": True,
                "expects_activate": False,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_set_active_script_special_cases(
        self, test_context: OrcaTestContext, case: dict
    ) -> None:
        """Test ScriptManager.set_active_script special cases."""
        essential_modules = self._setup_dependencies(test_context)
        default_script = essential_modules["default_script"]
        from orca.script_manager import ScriptManager

        manager = ScriptManager()
        old_script = test_context.Mock()
        old_script.deactivate = test_context.Mock()
        manager._active_script = old_script

        if case["new_script_type"] == "same":
            manager._active_script = default_script
            manager.set_active_script(default_script, "same script")
            if case["expects_deactivate"]:
                default_script.deactivate.assert_called_once()
            else:
                default_script.deactivate.assert_not_called()
                default_script.activate.assert_not_called()
        else:
            manager.set_active_script(None)
            if case["expects_deactivate"]:
                old_script.deactivate.assert_called_once()
            assert manager._active_script is None

    def test_set_active_script_runtime_settings(self, test_context: OrcaTestContext) -> None:
        """Test ScriptManager.set_active_script preserves runtime settings for same app."""

        essential_modules = self._setup_dependencies(test_context)
        default_script = essential_modules["default_script"]
        from orca.script_manager import ScriptManager

        settings_patch = "orca.script_manager.settings_manager.get_manager"
        mock_get_settings_manager = test_context.Mock()
        test_context.patch(settings_patch, new=mock_get_settings_manager)
        test_context.patch(
            "orca.script_manager.braille.checkBrailleSetting", return_value=None
        )
        test_context.patch(
            "orca.script_manager.braille.setupKeyRanges", return_value=None
        )
        speech_patch = "orca.script_manager.speech_and_verbosity_manager.get_manager"
        mock_get_speech_manager = test_context.Mock()
        test_context.patch(speech_patch, new=mock_get_speech_manager)
        manager_instance = test_context.Mock()
        runtime_settings = {"key1": "value1", "key2": "value2"}
        manager_instance.get_runtime_settings = test_context.Mock(return_value=runtime_settings)
        manager_instance.set_setting = test_context.Mock()
        mock_get_settings_manager.return_value = manager_instance

        speech_manager_instance = test_context.Mock()
        mock_get_speech_manager.return_value = speech_manager_instance
        manager = ScriptManager()
        same_app = test_context.Mock()
        old_script = test_context.Mock()
        old_script.app = same_app
        old_script.deactivate = test_context.Mock()
        new_script = default_script
        new_script.app = same_app
        new_script.activate = test_context.Mock()
        new_script.braille_bindings = {}
        manager._active_script = old_script
        manager.set_active_script(new_script)
        expected_calls = [
            call("key1", "value1"),
            call("key2", "value2"),
        ]
        manager_instance.set_setting.assert_has_calls(expected_calls, any_order=True)

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "mixed_apps_normal", "has_app_in_desktop": True, "has_key_error": False},
            {
                "id": "no_desktop_apps_with_key_error",
                "has_app_in_desktop": False,
                "has_key_error": True,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_reclaim_scripts(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test ScriptManager.reclaim_scripts with various scenarios."""
        self._setup_dependencies(test_context)
        from orca.script_manager import ScriptManager

        desktop_patch = "orca.script_manager.AXUtilities.is_application_in_desktop"
        mock_is_in_desktop = test_context.Mock()
        test_context.patch(desktop_patch, new=mock_is_in_desktop)
        manager = ScriptManager()
        app_not_in_desktop = test_context.Mock()

        if case["has_app_in_desktop"]:
            app_in_desktop = test_context.Mock()
            manager.app_scripts = {
                app_in_desktop: test_context.Mock(),
                app_not_in_desktop: test_context.Mock(),
            }
            manager.toolkit_scripts = {app_not_in_desktop: test_context.Mock()}
            manager.custom_scripts = {app_not_in_desktop: test_context.Mock()}
            manager._sleep_mode_scripts = {app_not_in_desktop: test_context.Mock()}

            def mock_is_in_desktop_func(app):
                return app == app_in_desktop

            mock_is_in_desktop.side_effect = mock_is_in_desktop_func
            manager.reclaim_scripts()
            assert app_in_desktop in manager.app_scripts
            assert app_not_in_desktop not in manager.app_scripts
            assert app_not_in_desktop not in manager.toolkit_scripts
            assert app_not_in_desktop not in manager.custom_scripts
            assert app_not_in_desktop not in manager._sleep_mode_scripts
        else:
            manager.app_scripts = {app_not_in_desktop: test_context.Mock()}
            if case["has_key_error"]:
                manager.toolkit_scripts = {}
                manager.custom_scripts = {}
                manager._sleep_mode_scripts = {}
            else:
                manager.toolkit_scripts = {app_not_in_desktop: test_context.Mock()}
                manager.custom_scripts = {app_not_in_desktop: test_context.Mock()}
                manager._sleep_mode_scripts = {app_not_in_desktop: test_context.Mock()}

            mock_is_in_desktop.return_value = False
            manager.reclaim_scripts()
            assert app_not_in_desktop not in manager.app_scripts
            if not case["has_key_error"]:
                assert app_not_in_desktop not in manager.toolkit_scripts
                assert app_not_in_desktop not in manager.custom_scripts
                assert app_not_in_desktop not in manager._sleep_mode_scripts

    def test_get_manager(self, test_context: OrcaTestContext) -> None:
        """Test script_manager.get_manager."""

        self._setup_dependencies(test_context)
        from orca import script_manager

        manager1 = script_manager.get_manager()
        manager2 = script_manager.get_manager()
        assert manager1 is manager2
        assert isinstance(manager1, script_manager.ScriptManager)



# Unit tests for speech_and_verbosity_manager.py methods.
#
# Copyright 2025 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=wrong-import-position
# pylint: disable=import-outside-toplevel
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-statements
# pylint: disable=protected-access
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=too-many-locals
# pylint: disable=too-many-lines

"""Unit tests for speech_and_verbosity_manager.py methods."""

from __future__ import annotations

import queue
from typing import TYPE_CHECKING

import pytest

if TYPE_CHECKING:
    from .orca_test_context import OrcaTestContext
    from unittest.mock import MagicMock


@pytest.mark.unit
class TestSpeechAndVerbosityManager:
    """Test SpeechAndVerbosityManager class methods."""

    def _setup_dependencies(self, test_context: OrcaTestContext) -> dict[str, MagicMock]:
        """Set up mocks for speech_and_verbosity_manager dependencies."""

        additional_modules = [
            "orca.mathsymbols",
            "orca.object_properties",
            "orca.pronunciation_dict",
            "orca.speech",
            "orca.speechserver",
            "orca.acss",
            "orca.ax_hypertext",
            "orca.ax_table",
            "orca.ax_text",
            "orca.ax_utilities",
            "orca.colornames",
            "orca.ax_utilities_text",
            "orca.ax_document",
        ]
        essential_modules = test_context.setup_shared_dependencies(additional_modules)

        settings_mock = essential_modules["orca.settings"]
        settings_mock.speechSystemOverride = "spiel"
        settings_mock.speechFactoryModules = ["spiel", "speechdispatcherfactory"]
        settings_mock.speechServerInfo = None
        settings_mock.voices = {}
        settings_mock.DEFAULT_VOICE = "default"

        # Set up constants for enum testing
        settings_mock.CAPITALIZATION_STYLE_NONE = "none"
        settings_mock.CAPITALIZATION_STYLE_SPELL = "spell"
        settings_mock.CAPITALIZATION_STYLE_ICON = "icon"
        settings_mock.PUNCTUATION_STYLE_NONE = 3
        settings_mock.PUNCTUATION_STYLE_SOME = 2
        settings_mock.PUNCTUATION_STYLE_MOST = 1
        settings_mock.PUNCTUATION_STYLE_ALL = 0
        settings_mock.VERBOSITY_LEVEL_BRIEF = 0
        settings_mock.VERBOSITY_LEVEL_VERBOSE = 1

        settings_manager_mock = essential_modules["orca.settings_manager"]
        settings_manager_instance = test_context.Mock()
        settings_manager_instance._prefs_dir = "/tmp/orca-test"
        settings_manager_instance.get_setting.return_value = True
        settings_manager_instance._load_user_customizations.return_value = True
        settings_manager_mock.get_manager.return_value = settings_manager_instance

        focus_manager_mock = essential_modules["orca.focus_manager"]
        focus_manager_mock.get_manager.return_value = test_context.Mock()

        dbus_service_mock = essential_modules["orca.dbus_service"]
        dbus_service_mock.get_remote_controller.return_value = test_context.Mock()

        def passthrough_decorator(func):
            return func

        dbus_service_mock.getter = passthrough_decorator
        dbus_service_mock.setter = passthrough_decorator
        dbus_service_mock.command = passthrough_decorator
        dbus_service_mock.parameterized_command = passthrough_decorator

        speech_mock = essential_modules["orca.speech"]
        speech_mock.get_speech_server.return_value = test_context.Mock()

        debug_mock = essential_modules["orca.debug"]
        debug_mock.print_message.return_value = None
        debug_mock.print_tokens.return_value = None

        acss_mock = essential_modules["orca.acss"]
        acss_mock.ACSS = test_context.Mock()
        acss_mock.ACSS.RATE = "rate"
        acss_mock.ACSS.AVERAGE_PITCH = "average-pitch"
        acss_mock.ACSS.GAIN = "gain"

        ax_hypertext_mock = essential_modules["orca.ax_hypertext"]
        ax_hypertext_mock.AXHypertext = test_context.Mock()
        ax_hypertext_mock.AXHypertext.get_all_links_in_range = test_context.Mock(return_value=[])
        ax_hypertext_mock.AXHypertext.get_link_end_offset = test_context.Mock(return_value=0)

        ax_object_mock = essential_modules["orca.ax_object"]
        ax_object_mock.AXObject = test_context.Mock()
        ax_object_mock.AXObject.find_ancestor_inclusive = test_context.Mock(return_value=None)

        ax_text_mock = essential_modules["orca.ax_text"]
        ax_text_mock.AXText = test_context.Mock()
        ax_text_mock.AXText.get_character_at_offset = test_context.Mock(return_value=("a", 0))

        orca_i18n_mock = essential_modules["orca.orca_i18n"]
        orca_i18n_mock._ = lambda x: x
        orca_i18n_mock.C_ = lambda c, x: x
        orca_i18n_mock.ngettext = lambda s, p, n: s if n == 1 else p

        return essential_modules

    def _setup_presentation_adjustment_mocks(
        self, test_context, manager, return_text="Hello world"
    ):
        """Set up common mocks for presentation adjustment testing."""
        test_context.patch_object(manager, "_adjust_for_links", return_value=return_text)
        test_context.patch_object(manager, "adjust_for_digits", return_value=return_text)
        test_context.patch_object(manager, "_adjust_for_repeats", return_value=return_text)
        test_context.patch_object(
            manager,
            "_adjust_for_verbalized_punctuation",
            return_value=return_text,
        )
        test_context.patch_object(
            manager, "_apply_pronunciation_dictionary", return_value=return_text
        )

    def test_init(self, test_context: OrcaTestContext) -> None:
        """Test get_handlers method."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        handlers = manager.get_handlers()
        assert handlers is not None

        handlers_refresh = manager.get_handlers(refresh=True)
        assert handlers_refresh is not None

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "server_none", "scenario": "none_server", "expected_result": None},
            {"id": "server_timeout", "scenario": "timeout", "expected_result": None},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_server_scenarios(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test _get_server method scenarios."""
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        if case["scenario"] == "none_server":
            essential_modules["orca.speech"].get_speech_server.return_value = None
        else:  # timeout scenario
            mock_speech_server = test_context.Mock()
            essential_modules["orca.speech"].get_speech_server.return_value = mock_speech_server

            def mock_queue_constructor():
                mock_queue = test_context.Mock()
                mock_queue.get.side_effect = queue.Empty()
                return mock_queue

            test_context.patch("queue.Queue", new=mock_queue_constructor)

        manager = SpeechAndVerbosityManager()
        result = manager._get_server()
        assert result is case["expected_result"]

    def test_get_available_servers(self, test_context: OrcaTestContext) -> None:
        """Test _get_server_module_map method with import errors."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        essential_modules["orca.settings"].speechFactoryModules = ["nonexistent_module"]

        def mock_import_module(name) -> None:
            raise ImportError("Module not found")

        test_context.patch("importlib.import_module", new=mock_import_module)
        manager = SpeechAndVerbosityManager()
        result = manager._get_server_module_map()
        assert not result

    def test_switch_server_invalid(self, test_context: OrcaTestContext) -> None:
        """Test _switch_server method with invalid server."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server_module_map", return_value={})
        result = manager._switch_server("Invalid Server")
        assert result is False

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "none", "server_return": None, "expected_result": ""},
            {"id": "with_server", "server_return": "mock_server", "expected_result": "Test Server"},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_current_server_scenarios(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test get_current_server method with various scenarios."""
        self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        if case["server_return"] == "mock_server":
            mock_server = test_context.Mock()
            mock_server.get_factory_name.return_value = "Test Server"
            test_context.patch_object(manager, "_get_server", return_value=mock_server)
        else:
            test_context.patch_object(manager, "_get_server", return_value=case["server_return"])

        result = manager.get_current_server()
        assert result == case["expected_result"]

    def test_get_current_synthesizer_none(self, test_context: OrcaTestContext) -> None:
        """Test get_current_synthesizer method when server is None."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=None)
        result = manager.get_current_synthesizer()
        assert result == ""

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "set_current_synthesizer",
                "method_name": "set_current_synthesizer",
                "method_args": ["espeak-ng"],
                "expected_result": False,
            },
            {
                "id": "get_available_synthesizers",
                "method_name": "get_available_synthesizers",
                "method_args": [],
                "expected_result": [],
            },
            {
                "id": "get_available_voices",
                "method_name": "get_available_voices",
                "method_args": [],
                "expected_result": [],
            },
            {
                "id": "get_voices_for_language",
                "method_name": "get_voices_for_language",
                "method_args": ["en"],
                "expected_result": [],
            },
            {
                "id": "get_current_voice",
                "method_name": "get_current_voice",
                "method_args": [],
                "expected_result": "",
            },
            {
                "id": "set_current_voice",
                "method_name": "set_current_voice",
                "method_args": ["en"],
                "expected_result": False,
            },
            {
                "id": "interrupt_speech",
                "method_name": "interrupt_speech",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "shutdown_speech",
                "method_name": "shutdown_speech",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "decrease_rate",
                "method_name": "decrease_rate",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "increase_rate",
                "method_name": "increase_rate",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "decrease_pitch",
                "method_name": "decrease_pitch",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "increase_pitch",
                "method_name": "increase_pitch",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "decrease_volume",
                "method_name": "decrease_volume",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "increase_volume",
                "method_name": "increase_volume",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "update_capitalization_style",
                "method_name": "update_capitalization_style",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "update_punctuation_level",
                "method_name": "update_punctuation_level",
                "method_args": [],
                "expected_result": True,
            },
            {
                "id": "update_synthesizer",
                "method_name": "update_synthesizer",
                "method_args": [],
                "expected_result": None,
            },
            {
                "id": "cycle_synthesizer",
                "method_name": "cycle_synthesizer",
                "method_args": [],
                "expected_result": True,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_methods_no_server(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test various methods when speech server is None."""
        self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=None)

        method = getattr(manager, case["method_name"])
        result = method(*case["method_args"])

        if case["expected_result"] is False:
            assert not result
        else:
            assert result == case["expected_result"]

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "speech_disabled",
                "speech_enabled": False,
                "expected_method": "shutdown_speech",
            },
            {"id": "speech_enabled", "speech_enabled": True, "expected_method": "start_speech"},
        ],
        ids=lambda case: case["id"],
    )
    def test_check_speech_setting(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test check_speech_setting method with various speech enabled settings."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["speech_enabled"]
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        mock_method = test_context.Mock()
        test_context.patch_object(manager, case["expected_method"], new=mock_method)
        manager.check_speech_setting()
        mock_method.assert_called_once()

    def test_start_speech(self, test_context: OrcaTestContext) -> None:
        """Test refresh_speech method."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        mock_shutdown = test_context.Mock()
        test_context.patch_object(manager, "shutdown_speech", new=mock_shutdown)
        mock_start = test_context.Mock()
        test_context.patch_object(manager, "start_speech", new=mock_start)
        result = manager.refresh_speech()
        assert result is True
        mock_shutdown.assert_called_once()
        mock_start.assert_called_once()

    def test_cycle_capitalization_style(self, test_context: OrcaTestContext) -> None:
        """Test cycle_capitalization_style method."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "update_capitalization_style", new=test_context.Mock())
        result = manager.cycle_capitalization_style()
        assert result is True

    def test_cycle_punctuation_level(self, test_context: OrcaTestContext) -> None:
        """Test change_number_style method."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        result = manager.change_number_style()
        assert result is True

    def test_toggle_speech(self, test_context: OrcaTestContext) -> None:
        """Test toggle_indentation_and_justification method."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        result = manager.toggle_indentation_and_justification()
        assert result is True

    def test_toggle_table_cell_reading_mode_no_script(self, test_context: OrcaTestContext) -> None:
        """Test _adjust_for_links static method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        essential_modules["orca.ax_hypertext"].get_all_links_in_range.return_value = []
        mock_obj = test_context.Mock()
        line = "Click here"
        start_offset = 0
        result = SpeechAndVerbosityManager._adjust_for_links(mock_obj, line, start_offset)
        assert isinstance(result, str)

    def test_adjust_for_repeats_static(self, test_context: OrcaTestContext) -> None:
        """Test _should_verbalize_punctuation static method returns False."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        essential_modules["orca.ax_object"].find_ancestor_inclusive.return_value = None
        mock_obj = test_context.Mock()
        result = SpeechAndVerbosityManager._should_verbalize_punctuation(mock_obj)
        assert result is False

    def test_get_indentation_description_disabled(self, test_context: OrcaTestContext) -> None:
        """Test get_indentation_description method when disabled."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.side_effect = lambda key: {
            "onlySpeakDisplayedText": True,
            "enableSpeechIndentation": False,
        }.get(key, False)
        manager = SpeechAndVerbosityManager()
        line = "    Hello world"
        result = manager.get_indentation_description(line)
        assert result == ""

    def test_get_error_description_disabled(self, test_context: OrcaTestContext) -> None:
        """Test get_manager function."""

        (self._setup_dependencies(test_context))
        from orca import speech_and_verbosity_manager

        manager1 = speech_and_verbosity_manager.get_manager()
        manager2 = speech_and_verbosity_manager.get_manager()

        assert manager1 is manager2
        assert isinstance(manager1, speech_and_verbosity_manager.SpeechAndVerbosityManager)

    def test_get_server_module_map_success(self, test_context: OrcaTestContext) -> None:
        """Test _get_server method with valid speech server."""

        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        mock_server = test_context.Mock()
        mock_server.is_alive.return_value = True
        test_context.patch(
            "orca.speech_and_verbosity_manager.speech.get_speech_server",
            side_effect=lambda: mock_server,
        )
        result = manager._get_server()
        assert result is mock_server

    def test_switch_server_success(self, test_context: OrcaTestContext) -> None:
        """Test get_current_synthesizer method with valid server."""

        mock_server = test_context.Mock()
        mock_server.get_output_module.return_value = "espeak-ng"
        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        result = manager.get_current_synthesizer()
        assert result == "espeak-ng"

    def test_set_current_synthesizer_with_server(self, test_context: OrcaTestContext) -> None:
        """Test get_available_voices method with valid server."""

        mock_voice1 = {"name": "Voice 1"}
        mock_voice2 = {"name": "Voice 2"}
        mock_voices = [mock_voice1, mock_voice2]
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        essential_modules["orca.speechserver"].VoiceFamily.NAME = "name"
        mock_server = test_context.Mock()
        mock_server.get_voice_families.return_value = mock_voices
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        result = manager.get_available_voices()
        assert result == ["Voice 1", "Voice 2"]

    def test_get_voices_for_language_with_server(self, test_context: OrcaTestContext) -> None:
        """Test set_current_voice method with valid server."""

        mock_voice = {"name": "Test Voice"}
        mock_voices = [mock_voice]
        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        essential_modules["orca.speechserver"].VoiceFamily.NAME = "name"
        mock_server = test_context.Mock()
        mock_server.get_voice_families.return_value = mock_voices
        mock_server.set_voice_family.return_value = True
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        test_context.patch_object(manager, "get_available_voices", return_value=["Test Voice"])
        result = manager.set_current_voice("Test Voice")
        assert result is True
        mock_server.set_voice_family.assert_called_once_with(mock_voice)

    def test_start_speech_with_server(self, test_context: OrcaTestContext) -> None:
        """Test shutdown_speech method with valid server."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        essential_modules["orca.speech"].shutdown_speech_server.return_value = True
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        result = manager.shutdown_speech()
        assert result is True

    def test_refresh_speech_with_server(self, test_context: OrcaTestContext) -> None:
        """Test set_rate method with valid value."""

        mock_voice = {"rate": 50}
        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        test_context.patch("orca.speech_and_verbosity_manager.ACSS.RATE", new="rate")
        test_context.patch(
            "orca.speech_and_verbosity_manager.settings.voices", new={"default": mock_voice}
        )
        test_context.patch(
            "orca.speech_and_verbosity_manager.settings.DEFAULT_VOICE", new="default"
        )
        result = manager.set_rate(75)
        assert result is True
        assert mock_voice["rate"] == 75

    def test_set_pitch_valid_value(self, test_context: OrcaTestContext) -> None:
        """Test set_volume method with valid value."""

        mock_voice = {"gain": 10.0}
        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        test_context.patch("orca.speech_and_verbosity_manager.ACSS.GAIN", new="gain")
        test_context.patch(
            "orca.speech_and_verbosity_manager.settings.voices", new={"default": mock_voice}
        )
        test_context.patch(
            "orca.speech_and_verbosity_manager.settings.DEFAULT_VOICE", new="default"
        )
        result = manager.set_volume(8.0)
        assert result is True
        assert mock_voice["gain"] == 8.0

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "decrease_rate",
                "method_name": "decrease_rate",
                "server_method": "decrease_speech_rate",
            },
            {
                "id": "increase_rate",
                "method_name": "increase_rate",
                "server_method": "increase_speech_rate",
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_rate_adjustment_with_server(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test rate adjustment methods with valid server."""

        mock_server = test_context.Mock()
        setattr(getattr(mock_server, case["server_method"]), "return_value", True)
        self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        result = getattr(manager, case["method_name"])()
        assert result is True
        getattr(mock_server, case["server_method"]).assert_called_once()

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "decrease_pitch",
                "method_name": "decrease_pitch",
                "server_method": "decrease_speech_pitch",
            },
            {
                "id": "increase_pitch",
                "method_name": "increase_pitch",
                "server_method": "increase_speech_pitch",
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_pitch_adjustment_with_server(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test pitch adjustment methods with valid server."""

        mock_server = test_context.Mock()
        setattr(getattr(mock_server, case["server_method"]), "return_value", True)
        self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        result = getattr(manager, case["method_name"])()
        assert result is True
        getattr(mock_server, case["server_method"]).assert_called_once()

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "decrease_volume",
                "method_name": "decrease_volume",
                "server_method": "decrease_speech_volume",
            },
            {
                "id": "increase_volume",
                "method_name": "increase_volume",
                "server_method": "increase_speech_volume",
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_volume_adjustment_with_server(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test volume adjustment methods with valid server."""

        mock_server = test_context.Mock()
        setattr(getattr(mock_server, case["server_method"]), "return_value", True)
        self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        result = getattr(manager, case["method_name"])()
        assert result is True
        getattr(mock_server, case["server_method"]).assert_called_once()

    def test_update_capitalization_style_with_server(self, test_context: OrcaTestContext) -> None:
        """Test update_synthesizer method with server and different synthesizer ID."""

        mock_server = test_context.Mock()
        mock_server.get_output_module.return_value = "espeak"
        mock_server.set_output_module.return_value = True
        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        test_context.patch(
            "orca.speech_and_verbosity_manager.settings.speechServerInfo",
            new=["Test Server", "espeak-ng"],
        )
        manager.update_synthesizer()
        mock_server.set_output_module.assert_called_once_with("espeak-ng")

    def test_cycle_synthesizer_with_server(self, test_context: OrcaTestContext) -> None:
        """Test cycle_synthesizer method with current not in available list."""

        mock_server = test_context.Mock()
        mock_server.list_output_modules.return_value = ["espeak", "festival"]
        mock_server.get_output_module.return_value = "unknown"
        mock_server.set_output_module.return_value = True
        (self._setup_dependencies(test_context))
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        test_context.patch_object(manager, "_get_server", return_value=mock_server)
        result = manager.cycle_synthesizer()
        assert result is True
        mock_server.set_output_module.assert_called_once_with("espeak")

    def test_apply_pronunciation_dictionary(self, test_context: OrcaTestContext) -> None:
        """Test _adjust_for_verbalized_punctuation static method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        mock_code_obj = test_context.Mock()
        (
            essential_modules["orca.ax_object"].AXObject.find_ancestor_inclusive.return_value
        ) = mock_code_obj
        essential_modules["orca.ax_document"].AXDocument.is_plain_text.return_value = False
        mock_obj = test_context.Mock()
        text = "Hello, world! How are you?"
        result = SpeechAndVerbosityManager._adjust_for_verbalized_punctuation(mock_obj, text)

        expected = "Hello ,  world !  How are you ? "
        assert result == expected

    def test_should_verbalize_punctuation_true(self, test_context: OrcaTestContext) -> None:
        """Test _should_verbalize_punctuation static method returns True."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        mock_code_obj = test_context.Mock()
        (
            essential_modules["orca.ax_object"].AXObject.find_ancestor_inclusive.return_value
        ) = mock_code_obj
        essential_modules["orca.ax_document"].AXDocument.is_plain_text.return_value = False
        mock_obj = test_context.Mock()
        result = SpeechAndVerbosityManager._should_verbalize_punctuation(mock_obj)
        assert result is True

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "get_capitalization_style", "setting_value": "spell", "expected": "spell"},
            {"id": "get_capitalization_style_none", "setting_value": "none", "expected": "none"},
            {"id": "get_capitalization_style_icon", "setting_value": "icon", "expected": "icon"},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_capitalization_style(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test get_capitalization_style method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["setting_value"]

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.get_capitalization_style()
        assert result == case["expected"]
        settings_manager_instance.get_setting.assert_called_with("capitalizationStyle")

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "set_capitalization_style_valid", "input_value": "spell", "expected": True},
            {"id": "set_capitalization_style_none", "input_value": "none", "expected": True},
            {"id": "set_capitalization_style_icon", "input_value": "icon", "expected": True},
            {"id": "set_capitalization_style_invalid", "input_value": "invalid", "expected": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_set_capitalization_style(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test set_capitalization_style method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        mock_update = test_context.Mock()
        test_context.patch_object(manager, "update_capitalization_style", new=mock_update)

        result = manager.set_capitalization_style(case["input_value"])
        assert result == case["expected"]

        if case["expected"]:
            settings_manager_instance.set_setting.assert_called_with(
                "capitalizationStyle", case["input_value"]
            )
            mock_update.assert_called_once()
        else:
            settings_manager_instance.set_setting.assert_not_called()
            mock_update.assert_not_called()

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "punctuation_none", "setting_value": 3, "expected": "none"},
            {"id": "punctuation_some", "setting_value": 2, "expected": "some"},
            {"id": "punctuation_most", "setting_value": 1, "expected": "most"},
            {"id": "punctuation_all", "setting_value": 0, "expected": "all"},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_punctuation_level(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test get_punctuation_level method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["setting_value"]

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.get_punctuation_level()
        assert result == case["expected"]
        settings_manager_instance.get_setting.assert_called_with("verbalizePunctuationStyle")

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "set_punctuation_none",
                "input_value": "none",
                "expected": True,
                "setting_int": 3,
            },
            {
                "id": "set_punctuation_some",
                "input_value": "some",
                "expected": True,
                "setting_int": 2,
            },
            {
                "id": "set_punctuation_most",
                "input_value": "most",
                "expected": True,
                "setting_int": 1,
            },
            {"id": "set_punctuation_all", "input_value": "all", "expected": True, "setting_int": 0},
            {
                "id": "set_punctuation_invalid",
                "input_value": "invalid",
                "expected": False,
                "setting_int": None,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_set_punctuation_level(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test set_punctuation_level method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()
        mock_update = test_context.Mock()
        test_context.patch_object(manager, "update_punctuation_level", new=mock_update)

        result = manager.set_punctuation_level(case["input_value"])
        assert result == case["expected"]

        if case["expected"]:
            settings_manager_instance.set_setting.assert_called_with(
                "verbalizePunctuationStyle", case["setting_int"]
            )
            mock_update.assert_called_once()
        else:
            settings_manager_instance.set_setting.assert_not_called()
            mock_update.assert_not_called()

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "verbosity_brief", "setting_value": 0, "expected": "brief"},
            {"id": "verbosity_verbose", "setting_value": 1, "expected": "verbose"},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_verbosity_level(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test get_verbosity_level method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["setting_value"]

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.get_verbosity_level()
        assert result == case["expected"]
        settings_manager_instance.get_setting.assert_called_with("speechVerbosityLevel")

    @pytest.mark.parametrize(
        "case",
        [
            {
                "id": "set_verbosity_brief",
                "input_value": "brief",
                "expected": True,
                "setting_int": 0,
            },
            {
                "id": "set_verbosity_verbose",
                "input_value": "verbose",
                "expected": True,
                "setting_int": 1,
            },
            {
                "id": "set_verbosity_invalid",
                "input_value": "invalid",
                "expected": False,
                "setting_int": None,
            },
        ],
        ids=lambda case: case["id"],
    )
    def test_set_verbosity_level(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test set_verbosity_level method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.set_verbosity_level(case["input_value"])
        assert result == case["expected"]

        if case["expected"]:
            settings_manager_instance.set_setting.assert_called_with(
                "speechVerbosityLevel", case["setting_int"]
            )
        else:
            settings_manager_instance.set_setting.assert_not_called()

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "speak_numbers_true", "setting_value": True, "expected": True},
            {"id": "speak_numbers_false", "setting_value": False, "expected": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_speak_numbers_as_digits(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test get_speak_numbers_as_digits method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["setting_value"]

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.get_speak_numbers_as_digits()
        assert result == case["expected"]
        settings_manager_instance.get_setting.assert_called_with("speakNumbersAsDigits")

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "set_speak_numbers_true", "input_value": True, "expected": True},
            {"id": "set_speak_numbers_false", "input_value": False, "expected": True},
        ],
        ids=lambda case: case["id"],
    )
    def test_set_speak_numbers_as_digits(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test set_speak_numbers_as_digits method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.set_speak_numbers_as_digits(case["input_value"])
        assert result == case["expected"]
        settings_manager_instance.set_setting.assert_called_with(
            "speakNumbersAsDigits", case["input_value"]
        )

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "speech_muted_true", "setting_value": True, "expected": True},
            {"id": "speech_muted_false", "setting_value": False, "expected": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_speech_is_muted(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test get_speech_is_muted method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["setting_value"]

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.get_speech_is_muted()
        assert result == case["expected"]
        settings_manager_instance.get_setting.assert_called_with("silenceSpeech")

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "set_speech_muted_true", "input_value": True, "expected": True},
            {"id": "set_speech_muted_false", "input_value": False, "expected": True},
        ],
        ids=lambda case: case["id"],
    )
    def test_set_speech_is_muted(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test set_speech_is_muted method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.set_speech_is_muted(case["input_value"])
        assert result == case["expected"]
        settings_manager_instance.set_setting.assert_called_with(
            "silenceSpeech", case["input_value"]
        )

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "only_speak_displayed_true", "setting_value": True, "expected": True},
            {"id": "only_speak_displayed_false", "setting_value": False, "expected": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_only_speak_displayed_text(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test get_only_speak_displayed_text method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["setting_value"]

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.get_only_speak_displayed_text()
        assert result == case["expected"]
        settings_manager_instance.get_setting.assert_called_with("onlySpeakDisplayedText")

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "set_only_speak_displayed_true", "input_value": True, "expected": True},
            {"id": "set_only_speak_displayed_false", "input_value": False, "expected": True},
        ],
        ids=lambda case: case["id"],
    )
    def test_set_only_speak_displayed_text(self, test_context: OrcaTestContext, case: dict) -> None:
        """Test set_only_speak_displayed_text method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.set_only_speak_displayed_text(case["input_value"])
        assert result == case["expected"]
        settings_manager_instance.set_setting.assert_called_with(
            "onlySpeakDisplayedText", case["input_value"]
        )

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "speak_indentation_true", "setting_value": True, "expected": True},
            {"id": "speak_indentation_false", "setting_value": False, "expected": False},
        ],
        ids=lambda case: case["id"],
    )
    def test_get_speak_indentation_and_justification(
        self, test_context: OrcaTestContext, case: dict
    ) -> None:
        """Test get_speak_indentation_and_justification method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value
        settings_manager_instance.get_setting.return_value = case["setting_value"]

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.get_speak_indentation_and_justification()
        assert result == case["expected"]
        settings_manager_instance.get_setting.assert_called_with("enableSpeechIndentation")

    @pytest.mark.parametrize(
        "case",
        [
            {"id": "set_speak_indentation_true", "input_value": True, "expected": True},
            {"id": "set_speak_indentation_false", "input_value": False, "expected": True},
        ],
        ids=lambda case: case["id"],
    )
    def test_set_speak_indentation_and_justification(
        self, test_context: OrcaTestContext, case: dict
    ) -> None:
        """Test set_speak_indentation_and_justification method."""

        essential_modules: dict[str, MagicMock] = self._setup_dependencies(test_context)
        settings_manager_instance = essential_modules[
            "orca.settings_manager"
        ].get_manager.return_value

        from orca.speech_and_verbosity_manager import SpeechAndVerbosityManager

        manager = SpeechAndVerbosityManager()

        result = manager.set_speak_indentation_and_justification(case["input_value"])
        assert result == case["expected"]
        settings_manager_instance.set_setting.assert_called_with(
            "enableSpeechIndentation", case["input_value"]
        )
